/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextField;

public class CGIAddAssassin implements CGICommand
{
   void addAssassin(CGI cgi, Player p, Map plugins, Map params) throws AssassinSQLException, IOException, StateBuilderException, CGIInvalidContentFormatException
   {
      // pass two, do plugin questions
      if ("2".equals(params.get("pass"))) {
         p = (Player) params.get("player");
         Event e = new Event();
         e.setActive(true);
         e.setEventTime(Tools.getNowTime());
         Assassin a = new Assassin();
         a.setPlayer(p);
         if ("on".equals(params.get("active")))
            e.addStateChange(new AssassinActivation(a));
         String s = (String) params.get("pseudonym");
         if (null != s && !"".equals(s)) {
            Pseudonym ps = new Pseudonym();
            ps.setValue(s);
            ps.setAssassin(a);
            e.addStateChange(new PseudonymGain(a, ps));
         }
         Assassin.commitAll();
         Pseudonym.commitAll();
         Event.commitAll();
         StateChange.commitAll();

         cgi.out("<h2>Add New Assassin</h2>");
         cgi.out("<div class=\"infomessage\">");
         cgi.out("   <p><b>Assassin Created</b></p>");
         cgi.out("   <p>Additional Questions:</p>");
         cgi.out("</div>");

         HTMLForm form = new HTMLForm("umpire.cgi?add-assassin", "Add");
         form.addField(new HiddenField("pass", "3"));
         if (Debug.debug) Debug.print(umpirecgi.class, "assassin: "+a);
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         lsb.registerInterest(a);
         Map myState=null;
         try {
            myState = lsb.generateState();
         } catch (StateBuilderException e1) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            e1.printStackTrace();
            throw new RuntimeException();
         }
         Map statev = Tools.viewerifyStateMap(myState);

         umpirecgi.currentAuth.clearSessionQuestions();
         Iterator plgs = Plugin.getPluginIterator(plugins);
         EventViewer ev = (EventViewer) e.getViewer();
         while (plgs.hasNext()) {
            Plugin pl = (Plugin) plgs.next();
            // EventViewer ev = (EventViewer) e.getViewer();
            AssassinViewer av = (AssassinViewer) a.getViewer();
            Vector qs = pl.assassinAdded(av, ev, statev, null);;
            Iterator qi = qs.iterator();
            for (int i=0; qi.hasNext(); i++) {
               Question q = (Question) qi.next();
               form.addField(CGITools.getPluginField(pl, q, i));
               umpirecgi.currentAuth.addSessionQuestion(pl, q);
            }
         }
         Assassin.commitAll();
         Event.commitAll();
         StateChange.commitAll();

         form.addField(new HiddenField("assassin", ""+a));
         form.addField(new HiddenField("event", ""+e));
         form.display(cgi);
      }

      // pass 3, commit everything
      else if ("3".equals(params.get("pass"))) {
         Assassin a = (Assassin) params.get("assassin");
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         lsb.registerInterest(a);
         Map myState=null;
         try {
            myState = lsb.generateState();
         } catch (StateBuilderException e1) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            e1.printStackTrace();
            throw new RuntimeException();
         }
         Map statev = Tools.viewerifyStateMap(myState);
         AssassinViewer av = (AssassinViewer) a.getViewer();
         EventViewer ev = (EventViewer) ((Event) params.get("event")).getViewer();

         // parse out plugin fields from POST
         Iterator plgs =  Plugin.getPluginIterator(plugins);
         while (plgs.hasNext()) {
            Plugin pl = (Plugin) plgs.next();
            Vector qs = CGITools.parseQuestions(pl, params);
            pl.assassinAdded(av, ev, statev, qs);
         }

         Event.commitAll();
         StateChange.commitAll();
         umpirecgi.currentAuth.clearSessionQuestions();

         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>New Assassin Added Successfully</b></p>");
         cgi.out("  <p>Additional questions completed.</p>");
         cgi.out("</div>");
         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-assassin,"+a.getAid()); } catch (CGIHeaderSentException CGIHSe) {} 
      }

      // pass one, do vanilla questions
      else {
         HTMLForm form = new HTMLForm("umpire.cgi?add-assassin", "Add");
         if (null == p) {
            if (Debug.debug) Debug.printMap(umpirecgi.class, params);
            if (null != params.get("1") && !"".equals(params.get("1"))){
               p = Player.getByPid( Integer.parseInt((String) params.get("1")) );
               form.addField(new DisplayField("Player:", p.getPrintable()));
               form.addField(new HiddenField("player",""+p));
            } else {
               List ps = Player.getAllPlayersOrderByName();
               form.addField(new CanonicalEntityField("player",
                        "Select Player to add assassin for:", ps, p));
            }
         } else  {
            form.addField(new DisplayField("Player:", p.getPrintable()));
            form.addField(new HiddenField("player", p.toString()));
         }
         form.addField(new CheckBox("active", "Make assassin active?", true));
         form.addField(new TextField("pseudonym", "Initial Pseudonym:", ""));
         form.addField(new HiddenField("pass", "2"));
         cgi.out("<h2>Add New Assassin</h3>");
         form.display(cgi);
      }
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException { addAssassin(cgi, null, plugins, params); } 
   public String getName() { return "Add Assassin"; }
   public String getDescription() { return "Give a player another instance in the game"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Assassin"; }
   public boolean isVisible() { return true; }

}

