/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextField;

public class CGIAddPseudonym implements CGICommand
{
   private void addPseudonym(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException , StateBuilderException, CGIInvalidContentFormatException
   {
      if (null == params.get("submit")) {
         cgi.out("<h2>Add Pseudonym</h2>");
         cgi.out("<p>");
         cgi.out("  Select Assassin and new pseudonym:");
         cgi.out("</p>");
         HTMLForm form = new HTMLForm("umpire.cgi?add-pseudonym", "Add");
         Assassin a = null;
         if (null != params.get("1") && !"".equals(params.get("1"))) {
            a = Assassin.getByAid(Integer.parseInt((String) params.get("1")));
            form.addField(new DisplayField("Assassin:", a.getPrintable()));
            form.addField(new HiddenField("assassin", ""+a));
         } else if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
            form.addField(new CanonicalEntityField("assassin","Assassin:",Assassin.getAssassinsOfPlayer((Player) umpirecgi.currentAuth.getPlayerViewer().getEntity()),a));
         else
            form.addField(new CanonicalEntityField("assassin","Assassin:",Assassin.getAllAssassinsOrderByRealName(),a));
         form.addField(new TextField("pseudonym", "New Pseudonym:", ""));
         form.display(cgi);
      }
      else {
         cgi.out("<h2>Add Pseudonym</h2>");

         String pseud = (String) params.get("pseudonym");
         if (pseud == null || "".equals(pseud)) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Blank Pseudonym</b></p>");
            cgi.out("  <p>Please go back and try again.</p>");
            cgi.out("</div>");
            return;
         }
         if(!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Assassin a = (Assassin) params.get("assassin");
            if (Debug.debug) Debug.print(umpirecgi.class, "Not umpire, requesting to change "+a+", pid="+a.getPlayer().getPid()+", user="+umpirecgi.currentAuth.getPlayerViewer().getEntity()+", user pid="+((Player)umpirecgi.currentAuth.getPlayerViewer().getEntity()).getPid());
            if (!a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("   <p><b>Insufficient Privileges</b></p>");
               cgi.out("   <p>Only the umpire can change other people's pseudonyms.</p>");
               cgi.out("</div>");
               return;
            }
         } 
         {
            cgi.out("<div class=\"donemessage\">");
            cgi.out("  <p><b>Pseudonym Added</b></p>");
            cgi.out("  <p>&nbsp;</p>");
            cgi.out("</div>");
            Assassin a = (Assassin) params.get("assassin");
            try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-assassin,"+a.getAid()); } catch (CGIHeaderSentException CGIHSe) {} 
            Pseudonym p = new Pseudonym();
            p.setValue(pseud);
            Event e = new Event();
            e.setActive(true);
            e.setEventTime(Tools.getNowTime());
            e.addStateChange(new PseudonymGain(a, p));
         }
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  addPseudonym(cgi, plugins, params); } 
   public String getName() { return "Add Pseudonym"; }
   public String getDescription() { return "Give an assassin another pseudonym"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Assassin"; }
   public boolean isVisible() {	return false; }
}

