/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.DropDown;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.NewTable;
import cx.ath.matthew.cgi.TextArea;
import cx.ath.matthew.cgi.TextField;

public class CGIEditAssassin implements CGICommand
{
   private void editAssassin(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   {
      cgi.out("<h2>Edit Assassin</h2>");
      cgi.flush();
      ((umpirecgi) cgi).setTab("Players");
      if (null != params.get("update")) {

         Assassin a = (Assassin) params.get("assassin");
         ((umpirecgi) cgi).setSelectedEntity(a.toString());
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         lsb.registerInterest(a);
         Map state=null;
         try {
            state = lsb.generateState();
         } catch (StateBuilderException e1) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            e1.printStackTrace();
            throw new RuntimeException();
         }
         AssassinState ast = (AssassinState) state.get(a);
         Player pl = a.getPlayer();

         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE) || (
                  umpirecgi.currentAuth.isAtLeast(AuthLevel.PLAYER) && 
                  a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity()))) {
            Event e = new Event();
            e.setActive(true);
            e.setEventTime(Tools.getNowTime());

            pl.setName((String) params.get("name"));
            pl.setEmail((String) params.get("email"));
            String college = (String) params.get("manualcol");
            if (null == college || "".equals(college)) college = (String) params.get("college");
            pl.setCollege(college);
            pl.setAddress((String) params.get("address"));
            pl.setWaterStatus((String) params.get("water"));
            pl.setNotes((String) params.get("notes"));

            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               // resurrect them
               if ("on".equals((String) params.get("alive")) && !ast.isAlive())
                  e.addStateChange(new Resurrection(a));
               // kill them
               if (!"on".equals((String) params.get("alive")) && ast.isAlive())
                  e.addStateChange(new Death(a, a));
               // deactivate them
               if (!"on".equals((String) params.get("active")) && ast.isActive())
                  e.addStateChange(new AssassinDeactivation(a));
               // activate them
               if ("on".equals((String) params.get("active")) && !ast.isActive())
                  e.addStateChange(new AssassinActivation(a));

               // plugin details
               List plugs = ast.getKnownPlugins();
               Iterator i = plugs.iterator();
               while (i.hasNext()) {
                  String p = (String) i.next();
                  if (Debug.debug) Debug.print(umpirecgi.class, "got "+p);
                  if (!"vanilla".equals(p)) {
                     Map m = ast.getState(p);
                     Iterator j = m.keySet().iterator();
                     while (j.hasNext()) {
                        String key = (String) j.next();
                        if (Debug.debug) Debug.print(umpirecgi.class, "got "+key);
                        Object cval = m.get(key);

                        // string
                        if (cval instanceof String) 
                           if (!cval.equals((String) params.get(p+"_"+key))) {
                              StateChange sc = new PluginStateChange(a,(Plugin) plugins.get(p), 
                                    key, (String) params.get(p+"_"+key));
                              e.addStateChange(sc);
                           }

                        // CanonicalEntity
                           else if (cval instanceof CanonicalEntity) {
                              StateChange sc = new PluginStateChange(a,(Plugin) plugins.get(p), 
                                    key, (CanonicalEntity) params.get(p+"_"+key));
                              e.addStateChange(sc);
                           }
                     }
                  }
               }
            }
            cgi.out("<div class=\"donemessage\">");
            cgi.out("  <p><b>Assassin Updated</b></p>");
            cgi.out("  <p>&nbsp;</p>");
            cgi.out("</div>");
         } else {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Cannot Edit Assassin</b></p>");
            cgi.out("  <p>You are not allowed to edit this assassin</p>");
            cgi.out("  <p>&nbsp;</p>");
            cgi.out("</div>");
         }
         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-assassin,"+a.getAid()); } catch (CGIHeaderSentException CGIHSe) {} 
      } 
      else if (null != params.get("setproperty")) {
         if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Insufficient Privileges</b></p>");
            cgi.out("  <p>Only the Umpire is allowed to set properties.</p>");
            cgi.out("</div>");
            return;
         }
         Assassin a = (Assassin) params.get("assassin");
         ((umpirecgi) cgi).setEntity(a.toString());
         Event e = new Event();
         e.setActive(true);
         e.setEventTime(Tools.getNowTime());
         StateChange sc = new PluginStateChange(a,(Plugin) plugins.get(params.get("plugin")), 
               (String) params.get("variable"), (String) params.get("value"));
         e.addStateChange(sc);
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Property Added</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-assassin,"+a.getAid()); } catch (CGIHeaderSentException CGIHSe) {} 
      }
      else {
         Assassin a = (Assassin) params.get("assassin");
         if (null == a) {
            String s = (String) params.get("1");
            if (null != s && !"".equals(s))
               a = Assassin.getByAid(Integer.parseInt(s));
         }
         if (null == a) {
            cgi.out("<p>");
            cgi.out("   Select the Assassin to edit.");
            cgi.out("</p>");
            HTMLForm form = new HTMLForm("umpire.cgi?edit-assassin", "Edit");
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
               form.addField(new CanonicalEntityField("assassin","Assassin:",Assassin.getAllAssassinsOrderByRealName(),a));
            else
               form.addField(new CanonicalEntityField("assassin","Assassin:",umpirecgi.currentAuth.getPlayerViewer().getAssassins(),a));
            form.display(cgi);
         } 
         else /*if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE) ||
               a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity()))*/ {
            boolean canedit;
            if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.LOGGED_IN)) canedit = false;
            else
               canedit = umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE) ||
                              a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity());
            ((umpirecgi) cgi).setSelectedEntity(a.toString());
            if (canedit) {
               cgi.out("<p>");
               cgi.out("   Edit the assassin details.");
               cgi.out("</p>");
            }
            LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
            lsb.registerInterest(a);
            Map state=null;
            try {
               state = lsb.generateState();
            } catch (StateBuilderException e1) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               e1.printStackTrace();
               throw new RuntimeException();
            }
            AssassinState ast = (AssassinState) state.get(a);
            AssassinStateViewer asv = (AssassinStateViewer) ast.getViewer();
            Player pl = a.getPlayer();
            cgi.out("<h3>Details for <span class='colour"+
                  Tools.getColourForAssassin(plugins, ast, pl.getName())+"'>"+
                  pl.getName()+" ("+a.getAid()+")</span></h3>");
            cgi.out("<div id=\"PseudonymOptions\">");
            cgi.out("<ul>");
            Iterator i = ast.getPseudonyms().iterator();
            while (i.hasNext()) {
               Pseudonym ps = (Pseudonym) i.next();
               cgi.out(" <li>a.k.a. <b>"+Tools.escapeXML(ps.getValue())+"</b>");
               if (canedit) {
                  cgi.out("[");
                  CGITools.printlink(cgi, "ps"+ps.getPsid(), "umpire.cgi?edit-pseudonym,"+ps.getPsid(),"Edit");    
                  cgi.out("]");
               }
               cgi.out("</li>");
            }
            if (canedit) {
               cgi.out(" <li>");
               CGITools.printlink(cgi, "addp", "umpire.cgi?add-pseudonym,"+a.getAid(), "Add Pseudonym");    
               cgi.out(" </li>");
            }
            cgi.out("</ul>");
            cgi.out("</div>");

            HTMLForm form = new HTMLForm("umpire.cgi?edit-assassin", "Edit");
            if (canedit) {
               form.addField(new TextField("name", "Real Name:", pl.getName()));
               form.addField(new TextField("email", "Email:", pl.getEmail()));
               String[] colleges = umpirecgi.getColleges();
               java.util.Arrays.sort(colleges);
               form.addField(new DropDown("college", "College:", colleges, pl.getCollege(), false));
               if (Arrays.binarySearch(colleges, pl.getCollege()) < 0) // not found
                  form.addField(new TextField("manualcol", "Enter college manually:", pl.getCollege()));
               else
                  form.addField(new TextField("manualcol", "Enter college manually:", ""));
               form.addField(new TextField("address", "Address:", pl.getAddress()));
               form.addField(new DropDown("water", "Water Status:", umpirecgi.waterstatuses, pl.getWaterStatus(), false));
               form.addField(new TextArea("notes", "Other Notes (will be sent to assassins):", pl.getNotes()));
            } else {
               form.addField(new DisplayField("Real Name:", pl.getName()));
               form.addField(new DisplayField("Email:", pl.getEmail()));
               form.addField(new DisplayField("College:", pl.getCollege()));
               form.addField(new DisplayField("Address:", pl.getAddress()));
               form.addField(new DisplayField("Water Status:", pl.getWaterStatus()));
               form.addField(new DisplayField("Other Notes (will be sent to assassins):", pl.getNotes()));
            }
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               form.addField(new CheckBox("active", "Active: ", ast.isActive()));
               form.addField(new CheckBox("alive", "Alive: ", ast.isAlive()));
            } else {
               form.addField(new DisplayField("Active: ", ast.isActive()?"yes":"no"));
               form.addField(new DisplayField("Alive: ", ast.isAlive()?"yes":"no"));
            }
            form.addField(new HiddenField("assassin", ""+a));
            form.addField(new HiddenField("update", "true"));
            
            // plugin details formatted nicely
            form.addField(new NewTable("Extra Details",""));
            i = plugins.values().iterator();
            while (i.hasNext()) {
               Plugin p = (Plugin) i.next();
               Map display = p.formatDetails(asv);
               Iterator di = display.keySet().iterator();
               while (di.hasNext()) {
                  String label = (String) di.next();
                  form.addField(new DisplayField(label, (String) display.get(label)));
               }
            }
            
            // plugin details
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               List plugs = ast.getKnownPlugins();
               i = plugs.iterator();
               while (i.hasNext()) {
                  String p = (String) i.next();
                  form.addField(new NewTable("Plugin",p));
                  if (Debug.debug) Debug.print(umpirecgi.class, "got "+p);
                  if (!"vanilla".equals(p)) {
                     form.addField(new DisplayField("Plugin", "<b>"+p+"</b>"));
                     Map m = ast.getState(p);
                     Iterator j = m.keySet().iterator();
                     while (j.hasNext()) {
                        String key = (String) j.next();
                        if (Debug.debug) Debug.print(umpirecgi.class, "got "+key);
                        Object cval = m.get(key);

                        // string
                        if (cval instanceof String) {
                           form.addField(new TextField(p+"_"+key ,key ,(String) cval));

                           // CanonicalEntity
                        } else if (cval instanceof CanonicalEntity) {
                           form.addField(new CanonicalEntityField(p+"_"+key,
                                    key, CanonicalEntity.getAll((CanonicalEntity) cval),
                                    (CanonicalEntity) cval));
                        }
                     }
                  }
               }
            }
            cgi.out("<div id=\"PlayerDetails\">");
            form.display(cgi);

            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               cgi.out("<p class='center'>");
               cgi.out("   Manually add a plugin value to an assassin:");
               cgi.out("</p>");
               form = new HTMLForm("umpire.cgi?edit-assassin", "Manual Add");
               form.addField(new DropDown("plugin", "Plugin:", new Vector(plugins.keySet()), "", false));
               form.addField(new TextField("variable", "Field:", ""));
               form.addField(new TextField("value", "Value:", ""));
               form.addField(new HiddenField("setproperty", "true"));
               form.addField(new HiddenField("assassin", ""+a));
               form.display(cgi);
            }
            cgi.out("</div>"); /* player details */

            cgi.out("<div id=\"EventHistory\">");
            cgi.out("<h4>Event History</h4>");
            cgi.out("<ul>");
            i = Event.getEventsInvolvingAssassinWithHeadline(a).iterator();
            while (i.hasNext()) {
               Event e = (Event) i.next();
               cgi.out("<li>");
               CGITools.printlink(cgi, "e"+e.getEid(), "umpire.cgi?edit-event,"+e.getEid(), e.getPrintable());
               cgi.out("</li>");
            }
            cgi.out("</ul>");
            cgi.out("</div>"); /* Event History */
            cgi.out("<h4>Reports from this assassin</h4>");
            cgi.out("<ul>");
            i = Report.getReportsBy(a).iterator();
            while (i.hasNext()) {
               Report r = (Report) i.next();
               cgi.out("<li>");
               if (canedit) 
                  CGITools.printlink(cgi, "r"+r.getRid(), "umpire.cgi?edit-report,"+r.getRid(), r.getPrintable());
               else
                  cgi.out(r.getPrintable());
               cgi.out("</li>");
            }
            cgi.out("</ul>");
         }
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  editAssassin(cgi, plugins, params); }
   public String getName() { return "Edit Assassin"; }
   public String getDescription() { return "Modify state values on an assassin, and change a players personal details"; }
   public AuthLevel accessLevel () {
      String level = Config.getConfig("vanilla", "edit-assassin-auth");
      if ("public".equals(level))
         return AuthLevel.PUBLIC; 
      if ("player".equals(level))
         return AuthLevel.PLAYER; 
      if ("umpire".equals(level))
         return AuthLevel.UMPIRE; 
      return AuthLevel.PLAYER; 
   }
   public String getSection() { return "Assassin"; }
   public boolean isVisible() {	return true; }
}

