/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextArea;

public class CGIEditReport implements CGICommand
{
   private void editReport(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   { 
      Debug.printMap(umpirecgi.class, params);
      if ("2".equals(params.get("pass")) || (null != params.get("1") && !"".equals(params.get("1")))) {
         HTMLForm form = new HTMLForm("umpire.cgi?edit-report", "Edit");
         Report r = (Report) params.get("report");
         if (null == r) r = Report.getByRid(Integer.parseInt((String) params.get("1")));
         Event e = r.getEvent();
         if (Debug.debug) Debug.print(umpirecgi.class, "report: "+r);
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         Assassin a = r.getAssassin();
         Map myState = null;
         if (null != a) {
            lsb.registerInterest(a);
            try {
               myState = lsb.generateState();
            } catch (StateBuilderException sbe) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               throw new RuntimeException();
            }
         }
         else
            myState = new HashMap();
         cgi.out("<h2>Edit Report</h2>");
         cgi.out("<p>");
         if (a != null)
            cgi.out("  Report by <span class=\"playername\">"+a.getPlayer().getName()+"</span>:");
         else
            cgi.out("  Comment:");
         cgi.out("</p>");
         cgi.out("<p>");
         cgi.out("   You can include XHTML in your report, but please remember");
         cgi.out("   to match all opening tags with closing tags, and escape");
         cgi.out("   &amp; with &amp;amp; and &gt;/&lt; with &amp;gt;/&amp;lt;");
         cgi.out("   if they are not in tags. Also, remember that all non-ascii");
         cgi.out("   characters (accents, etc) should be entered in the UTF-8");
         cgi.out("   encoding. HTML Entities (eg &amp;eacute;) can be entered as");
         cgi.out("   &lt;entity name='eacute'/&gt;. ");
         cgi.out("   Reports may also contain the special sequences ");
         cgi.out("   [Pn], [Nn] and [Ln], ");
         cgi.out("   where n is the participant number. ");
         cgi.out("   [Pn] will be replaced by their pseudonym, ");
         cgi.out("   [Nn] by their name ");
         cgi.out("   and [Ln] by the list of all their pseudonyms. ");
         cgi.out("</p>");
         
         LightweightStateBuilder lsb2 = new LightweightStateBuilder(e.getEventTime());
         Iterator i = e.getParticipants().values().iterator();
         while (i.hasNext()) 
            lsb2.registerInterest((Assassin) i.next());
         Map state=null;
         try {
            state = lsb2.generateState();
         } catch (StateBuilderException e1) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            e1.printStackTrace();
            throw new RuntimeException();
         }
         
         CGIEditEvent.printParticipants(e,cgi,state,plugins);
         form.addField(new HiddenField("pass","3"));
         form.addField(new HiddenField("report",""+r));
         if (a != null) {
            Pseudonym p = r.getPseudonym();
            form.addField(new CanonicalEntityField("pseudonym","Choose Pseudonym:",((AssassinState) myState.get(r.getAssassin())).getPseudonyms(), p));
            form.addField(new CheckBox("nopseud", "Use Real Name", null == p));
         }
         form.addField(new TextArea("text","Report:",r.getText(), 80, 25));
         form.display(cgi);
      } else if ("3".equals(params.get("pass"))) {
         // Done asking questions, so update event
         cgi.out("<h2>Edit Report</h2>");
         Report r = (Report) params.get("report");
         if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Assassin a = r.getAssassin();
            if (null == a || 
                  !a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("   <p><b>Insufficient Privileges</b></p>");
               cgi.out("   <p>I'm sorry, only the umpire can edit other people's reports.</p>");
               cgi.out("</div>");
               return;
            }
         }
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Report Updated</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
         Pseudonym pseudonym = (Pseudonym) params.get("pseudonym");
         String text = (String) params.get("text");
         if ("on".equals(params.get("nopseud")))
            r.setPseudonym(null);
         else
            r.setPseudonym(pseudonym);
         r.setText(text);
         try { cgi.header("Refresh", "2; URL=umpire.cgi"); } catch (CGIHeaderSentException CGIHSe) {} 
         CGIGeneratePages.generatePages(cgi, plugins, params, false);
         Report.commitAll();
      } else {
         HTMLForm form = new HTMLForm("umpire.cgi?edit-report", "Edit");
         cgi.out("<h2>Edit Report</h2>");
         cgi.out("<p>");
         cgi.out("  Select Report to edit:");
         cgi.out("</p>");
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(
                  new CanonicalEntityField(
                     "report",
                     "Report:",
                     Report.getAllReports(),
                     (CanonicalEntity) null
                     ));
         } else {
            form.addField(
                  new CanonicalEntityField(
                     "report",
                     "Report:",
                     Report.getReportsBy(
                        (Player) umpirecgi.currentAuth.getPlayerViewer().getEntity()
                        ),
                     (CanonicalEntity) null
                     ));
         }
         form.addField(new HiddenField("pass","2"));
         form.display(cgi);
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  editReport(cgi, plugins, params); } 
   public String getName() { return "Edit Report"; }
   public String getDescription() { return "Edit a player's report"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return false; }
}

