/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.exceptions.XMLParseException;
import org.ucam.srcf.assassins.xml.Config;
import org.ucam.srcf.assassins.xml.Handler;
import org.ucam.srcf.assassins.xml.PageHandler;
import org.ucam.srcf.assassins.xml.WebPageParser;
import org.ucam.srcf.assassins.xml.WebPageParserSAX;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;


public class CGIGeneratePages implements CGICommand
{
   public static void generatePages(CGI cgi, Map plugins, Map params, boolean verbose)  throws AssassinSQLException, IOException, StateBuilderException , CGIInvalidContentFormatException
   { 
      if (!verbose)  { 
         String mode = Config.getConfig("vanilla", "update-page-on-commands");
         if ("player".equals(mode)) {
            if (!(umpirecgi.currentAuth.isAtLeast(AuthLevel.PLAYER) &&  
                  !umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))) return;
         } else if ("umpire".equals(mode)) {
            if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) return;
         } else if ("all".equals(mode)){
            if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.LOGGED_IN)) return;
         } else return;
      }
      if (verbose) {
         cgi.out("<h2>Generating News and List Pages</h2>");
         cgi.out("<div class=\"infomessage\">");
         cgi.out("  <p><b>Generating...</b></p>");
         cgi.out("  <p>The news pages and player listings are being generated now...</p>");
         cgi.out("</div>");
      }
      cgi.flush();
      
      ByteArrayOutputStream outb = new ByteArrayOutputStream();
     
      if (verbose) {
         cgi.out("<pre>");
      }
      if (Debug.debug) Debug.print(umpirecgi.class,"Building state...");
      StateBuilder sb = new StateBuilder(plugins);
      Map state = sb.doNews(Tools.getNowTime(), outb);
      if (Debug.debug) Debug.print(umpirecgi.class,"State built.");
      boolean errors = false;
      
      ByteArrayInputStream inb = new ByteArrayInputStream(outb.toByteArray());
      BufferedReader inr = new BufferedReader(new InputStreamReader(inb));
      
      if(inr.ready()) errors = true;
      if(verbose){
	      // output any xslt errors
    	  while (inr.ready()) cgi.out(inr.readLine());
      }
      String path = Config.getConfig("vanilla","path-to-pages")+"/";
      // need to run xsltproc on all the news....
      String xslt_template = Config.getConfig("vanilla","xsl-file");

      /* Iterating over all the desired templates */
      int i = 1;
      String myTemplate = Config.getConfig("vanilla","template-" + i);
      while (null != myTemplate) {
         String out = Config.getConfig("vanilla","template-page-name-" + i);
         if (Debug.debug) Debug.print(umpirecgi.class, "Path "+path+", Template "+myTemplate+", out "+out);

         // generate the XML
         try {
            String target = path+out+".xml";
            FileOutputStream fos = new FileOutputStream(new File(target));
            Handler ph = new PageHandler(fos,out,state,plugins,sb.getPages());
            WebPageParser wl = new WebPageParserSAX(myTemplate,ph);
            wl.generate();
         } catch (XMLParseException XMLPe) {
            if(Debug.debug) Debug.print(umpirecgi.class, "XML parse error on file "+path+out+".xml");
            throw new IOException(XMLPe.getMessage());
         }

         // convert to HTML
         if (Debug.debug) Debug.print(umpirecgi.class,"exec(/usr/bin/xsltproc -o "+path+out+".html "+xslt_template+" "+path+out+".xml)");
         
         outb.reset();
         
         int result = Tools.exec(new String[] {
            "/usr/bin/xsltproc",
            "-o",
            path+out+".html",
            xslt_template,
            path+out+".xml"}, outb);
         if(result!=0) errors=true;
         
         inb = new ByteArrayInputStream(outb.toByteArray());
         inr = new BufferedReader(new InputStreamReader(inb));
         
         // output any xslt errors
         if (verbose) {
        	 if(result!=0){
 	     		cgi.out("There was an error in the XML for " + out);
 	    	 }
             while (inr.ready()) cgi.out(inr.readLine());
         }
         myTemplate = Config.getConfig("vanilla","template-" + ++i);
      }
      if(verbose){
    	  cgi.out("</pre>");
	      if(!errors){
		      cgi.out("<div class=\"donemessage\">");
		      cgi.out("   <p><b>Done</b></p>");
		      cgi.out("   <p>Please remember to check the pages for any mistakes or information leaks.</p>");
		      cgi.out("   <p>The pages are available <a href='"+path+"'>here</a>.</p>");
		      cgi.out("</div>");
	      }
	      else{
	    	  cgi.out("<div class=\"errormessage\">");
		      cgi.out("   <p><b>Error Generating Pages</b></p>");
		      cgi.out("   <p>Please check for reports with invalid XHTML (failing to close tags, not escaping &lt;, &gt;, &amp, etc.), and other XML problems.</p>");
		      cgi.out("   <p>The pages are available <a href='"+path+"'>here</a>.</p>");
		      cgi.out("</div>");
	      }
      }

      return; 
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  generatePages(cgi, plugins, params, true); } 
   public String getName() { return "Generate Pages"; }
   public String getDescription() { return "Generates the news and player list pages"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Updates"; }
   public boolean isVisible() { return true; }

}

