/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.HiddenField;

public class CGIRemoveReport implements CGICommand
{
   private void removeReport(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   { 
      Debug.printMap(umpirecgi.class, params);
      if ("2".equals(params.get("pass")) || (null != params.get("1") && !"".equals(params.get("1")))) {
         HTMLForm form = new HTMLForm("umpire.cgi?remove-report", "Delete Report?");
         Report r = (Report) params.get("report");
         if (null == r) r = Report.getByRid(Integer.parseInt((String) params.get("1")));
         if (Debug.debug) Debug.print(umpirecgi.class, "report: "+r);
         Assassin a = r.getAssassin();
         cgi.out("<h2>Delete Report</h2>");
         cgi.out("<div class=\"warningmessage\">");
         cgi.out("   <p><b>Are You Sure About This?</b></p>");
         cgi.out("   <p>This will delete the");
         if (a != null)
            cgi.out(" report by <span class=\"playername\">"+a.getPlayer().getName()+"</span> ");
         else
            cgi.out(" comment ");
         cgi.out("on event <span class=\"reportheadline\">"+r.getEvent().getReplaceHeadline()+"</span>!");
         cgi.out("</p>");
         cgi.out("</div>");
         form.addField(new HiddenField("pass","3"));
         form.addField(new HiddenField("report",""+r));
         form.addField(new HiddenField("event",""+r.getEvent()));
         form.display(cgi);
      } else if ("3".equals(params.get("pass"))) {
         // Done asking questions, so update event
         cgi.out("<h2>Delete Report</h2>");
         Report r = (Report) params.get("report");
         Event e = (Event) params.get("event");
         if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Assassin a = r.getAssassin();
            if (null == a || 
                  !a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("   <p><b>Insufficient Privileges</b></p>");
               cgi.out("   <p>I'm sorry, only the umpire can delete other people's reports.</p>");
               cgi.out("</div>");
               return;
            }
         }
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Report Permanently Deleted</b></p>");
         cgi.out("  <p>I really hope you wanted to do that.</p>");
         cgi.out("</div>");
         e.removeReport(r);
         try { cgi.header("Refresh", "2; URL=umpire.cgi"); } catch (CGIHeaderSentException CGIHSe) {} 
         Report.commitAll();
         Event.commitAll();
         CGIGeneratePages.generatePages(cgi, plugins, params, false);
      } else {
         HTMLForm form = new HTMLForm("umpire.cgi?remove-report", "Delete");
         cgi.out("<h2>Delete Report</h2>");
         cgi.out("<p>");
         cgi.out("  Select Report to delete:");
         cgi.out("</p>");
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(
                  new CanonicalEntityField(
                     "report",
                     "Report:",
                     Report.getAllReports(),
                     (CanonicalEntity) null
                     ));
         } else {
            form.addField(
                  new CanonicalEntityField(
                     "report",
                     "Report:",
                     Report.getReportsBy(
                        (Player) umpirecgi.currentAuth.getPlayerViewer().getEntity()
                        ),
                     (CanonicalEntity) null
                     ));
         }
         form.addField(new HiddenField("pass","2"));
         form.display(cgi);
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  removeReport(cgi, plugins, params); } 
   public String getName() { return "Remove Report"; }
   public String getDescription() { return "Delete a player's report"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return false; }
}

