/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.SubmitButton;

public class CGIRemoveStateChange implements CGICommand
{

   private void removeStateChange(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   {
      // 0 = select event, 1 = select participant, 2 = select pseudonym, 3 = commit
      if (null != params.get("reset")) {
         umpirecgi.currentAuth.unsetData("cpp_event");
         umpirecgi.currentAuth.unsetData("cpp_statechange");
      }
      cgi.out("<h2>Remove a StateChange from an Event</h2>");
      cgi.flush();
      int pass = 0;
      Event e = null;
      if (null !=  params.get("1") && !"".equals(params.get("1")))
         e = Event.getByID(Integer.parseInt((String) params.get("1")));
      if (null != umpirecgi.currentAuth.getData("cpp_event"))
         e = Event.getByID(Integer.parseInt(umpirecgi.currentAuth.getData("cpp_event")));
      if (null != params.get("event")) 
         e = (Event) params.get("event");
      StateChange sc = null;
      if (null !=  params.get("2") && !"".equals(params.get("2")))
         sc = StateChange.getByScid(Integer.parseInt((String) params.get("2")));
      if (null != params.get("statechange")) 
         sc = (StateChange) params.get("statechange");
      if (null != umpirecgi.currentAuth.getData("cpp_statechange"))
         sc = StateChange.getByScid(Integer.parseInt(umpirecgi.currentAuth.getData("cpp_statechange")));

      HTMLForm form = new HTMLForm("umpire.cgi?remove-statechange", "Remove StateChange");

      if (null == e) pass = 0;
      else if (null == sc) pass = 1;
      else if (null == params.get("confirm")) pass = 2;
      else pass = 3;

      switch (pass) {
         case 0:
            cgi.out("<p>");
            cgi.out("  Select Event:");
            cgi.out("</p>");
            List l = Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(new CanonicalEntityField("event","Event:",l,(CanonicalEntity) null));
            form.display(cgi);
            break;
         case 1:
            Collection statechanges = e.getStateChanges();
            cgi.out("<p>");
            cgi.out("  Select StateChange:");
            cgi.out("</p>");
            form.addField(new DisplayField("Event:", e.getPrintable()));
            umpirecgi.currentAuth.storeData("cpp_event", ""+e.getEid());
            form.addField(new CanonicalEntityField("statechange","StateChange:",
                     statechanges,(CanonicalEntity)null));
            form.addField(new SubmitButton("reset", "Reset"));
            form.display(cgi);
            break;
         case 2:
            cgi.out("<p>");
            cgi.out("  <b>Are you <em>really</em> sure that you want to remove this state change?</b>");
            cgi.out("</p>");
            form.addField(new DisplayField("Event:", e.getPrintable()));
            umpirecgi.currentAuth.storeData("cpp_event", ""+e.getEid());
            umpirecgi.currentAuth.storeData("cpp_statechange", ""+sc.getScid());
            form.addField(new DisplayField("StateChange:",sc.getPrintable()));
            form.addField(new HiddenField("confirm","true"));
            form.addField(new SubmitButton("reset", "Reset"));
            form.display(cgi);
            break;
         case 3:
            try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
            e.removeStateChange(sc);
            StateChange.commitAll();
            Event.commitAll();
            umpirecgi.currentAuth.unsetData("cpp_event");
            umpirecgi.currentAuth.unsetData("cpp_statechange");
            cgi.out("<div class=\"donemessage\">");
            cgi.out("<p><b>StateChange Successfully Removed</b></p>");
            cgi.out("<p>&nbsp;</p>");
            cgi.out("</div>");
            cgi.out("<div class=\"warningmessage\">");
            cgi.out("<p><b>Run Update-State Immediately!</b></p>");
            cgi.out("<p>We do not guarantee the state of anything after removing a state change, it is important to run update-state <b>NOW</b> to fix things</p>");
            cgi.out("<p>&nbsp;</p>");
            cgi.out("</div>");
            break;
      }
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException { removeStateChange(cgi, plugins, params); } 
   public String getName() { return "Remove State Change"; }
   public String getDescription() { return "Removes a State Change associated with an event"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }
}

