/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

/**
 * Class to build state implemented using a cache.
 * Should be one cache x time units in the past, we update it 
 * whenever we rebuild state over this time value. This should
 * lead to a constant algorithm for building state in most cases
 * rather than an O(n) one for n the number of events.
 * Works by in most cases we don't edit anything older than x days
 * or whatever
 * <b>FIXME: Write the rest of this class before using it!</b>
 */
public class CachedStateBuilder extends StateBuilder
{
	private Map cached;
	private long cached_time;
	private long new_cache_time;

   public CachedStateBuilder(Map p)
   {
		super(p);
		/*
		 * FIXME: Read a config file value to determine how far in 
		 * the past the cache is supposed to be, for example 3 days 
		 * sounds good. to be fiddled with probably.
		 * Put the obvious in new_cache_time
		 */
		readCache();
   }

	/**
	 * Reads the cache from the database.
	 * If there isn't one then make cached a fresh map a la the setup
	 * in Statebuilder and set cached_time to the start of the game
	 */
	private void readCache() {
		/*
		 * FIXME: I need writing, put the cache in the Map cached,
		 * and the time of the cache values into long cached_time
		 */
	}

	private void writeCache(Map newcache, long newtime) {
		/*
		 * FIXME: I need writing, write the current cache and associated time
		 * to disk
		 */
	}
   
   // FIXME: doNews is seperate now!
   Map buildStateAtTime(long time, boolean doNews) throws StateBuilderException
   {
		/* FIXME: the line below, all news gen has just been commented for the tiem being */
		if(doNews)
			throw new StateBuilderException("CachedStateBuilder doens't do the news. Fixme please.");
		if(time<cached_time) {
			// FIXME: return super.buildStateAtTime(time, doNews);
		}

      try
      {
         if (Debug.debug) Debug.print(StateBuilder.class, "buildStateAtTime");
			/* deep copy the cache into a new map */
         Map states = new HashMap();
         Iterator i = cached.keySet().iterator();
         while (i.hasNext())
         {
            Assassin a = (Assassin) i.next();
            AssassinState s = ((AssassinState) cached.get(a)).deepClone();
            states.put(a, s);
         }
/*       NewsCreator nc = null;
 		 if (doNews) {
            if (Debug.debug) Debug.print(StateBuilder.class, "instantiating NewsCreator");
				try {
	            nc = new NewsCreator(Config.getConfig("vanilla","path-to-pages"), plugins);
				} catch (java.io.IOException ioe) {
					throw new StateBuilderException(ioe);
				}
            if (Debug.debug) Debug.print(StateBuilder.class, "instantiated ok");
         }*/
         StateChange.preCache(); /*FIXME: only need the ones after a 
											  specific time caching? */
         List events = Event.getEventsInTime(cached_time, time);
         // IMPORTANT: I'm assuming here that getAllEvents() returns them
         // in chronological order! otherwise this will fail horribly!
         i = events.iterator();
         if (Debug.debug) Debug.print(StateBuilder.class, "applying events");
			long last_event_time=cached_time;
         while (i.hasNext())
         {
            Event e = (Event) i.next();
				/* Update the cache when we're in a position to */
				if(	last_event_time < new_cache_time &&
						e.getEventTime() > new_cache_time
						)
					writeCache(states, last_event_time);
						
            if (Debug.debug) Debug.print(StateBuilder.class, e.toString());
            if (e.getEventTime() > time) { continue; } // Is this right?
            if (e.isActive())
            {
               List sclist = e.getStateChanges();
               Iterator sci = sclist.iterator();
               while (sci.hasNext())
               {
                  StateChange sc = (StateChange) sci.next();
                  Assassin a = sc.getAssassin();
                  String pname = sc.getPluginName();
                  String field = sc.getFieldName();
                  Object value = sc.getValue();
                  if (Debug.debug) Debug.print(StateBuilder.class, "StateChange: "+sc);
                  // special cases
                  if (sc instanceof PseudonymGain)
                  {
                     if (Debug.debug) Debug.print(StateBuilder.class, "Pseudonym: "+ value.getClass());
                     if (Debug.debug) Debug.print(StateBuilder.class, "AssassinState: "+states.get(a));
                     ((AssassinState)states.get(a)).addPseudonym((Pseudonym) value);
                     if (Debug.debug) Debug.print(StateBuilder.class, "Done");
                  }
                  else if (sc instanceof PseudonymLoss)
                  {
                     ((AssassinState)states.get(a)).removePseudonym((Pseudonym) value);
                  }
                  else
                  {
                     if (sc instanceof Death)
                     {
                        field = "alive";
                        value = "false";
                     }
                     else if (sc instanceof Resurrection)
                     {
                        field = "alive";
                        value = "true";
                     }
                     ((AssassinState)states.get(a)).setProperty(pname,field,value);
                  }
               }
/*               if (Debug.debug) Debug.print(StateBuilder.class, "Beginning event pass to NewsCreator");
               if (doNews && e.isVisible())
               {
                  Map part = new HashMap();
                  Iterator pi = ((e.getParticipants()).values()).iterator();
                  if (Debug.debug) Debug.print(StateBuilder.class, "About to start iterating");
                  while (pi.hasNext())
                  {
                     Assassin p = (Assassin) pi.next();
                     AssassinState ps = (AssassinState) states.get(p);
                     // Over Christmas 2003, mac57 assured amgb2 that a
                     // method AssassinState.deepClone() would never be
                     // needed. Less than two days before the start of the
                     // Michaelmas 2004 game, while frantically trying to
                     // fix a bug in mac57's code, amgb2 discovered that he
                     // had been wrong.
                     if (null == ps)
                     {
                        part.put(p, null);
                     }
                     else
                     {
                        part.put(p, ps.deepClone());
                     }
                  }
                  if (Debug.debug) Debug.print(StateBuilder.class, "done iterating");
                  if (Debug.debug) Debug.print(StateBuilder.class, "about to add event to NewsCreator");
                  nc.addEvent(e, part);
               }*/
               if (Debug.debug) Debug.print(StateBuilder.class, "Events passed");
            }
         }
 /*        if (doNews)
         {
            if (Debug.debug) Debug.print(StateBuilder.class, "generating news pages");
            nc.generate();
         }*/
         return states;
      }
      catch (AssassinSQLException ASe)
      {
         throw new StateBuilderException(ASe);
      }
      /*catch (ParseException Pe)
      {
         throw new StateBuilderException(Pe);
      }*/
   }
}
