/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.Map;

import org.ucam.srcf.assassins.xml.Config;

/**
 * Used to perform debug output.
 *
 * E.G.:
 *
 * <code>
 *    if (Debug.debug) Debug.print(Player.class, "Some Debug Output");
 * </code>
 */
public class Debug
{
   public static final boolean debug = false;
   public static PrintStream debugout = System.err;

   private static boolean debugging(Class c)
   {
      if (debug) {
         try {
            if (null == c) return true;
            String d = Config.getConfig("debug", c.getName());
            if ("true".equals(d)) return true;
            return false;
         } catch (NullPointerException NPe) { return false; }
      }
      return false;
   }

   public static void setOutput(String filename) throws IOException
   {
      debugout = new PrintStream(new FileOutputStream(filename, true));
   }

   public static void setOutput() throws IOException {
      String filename = Config.getConfig("debug", "log-path");
      if(filename == null || filename == "")
         filename = "./debug.log";
      setOutput(filename);
   }
   
   public static void print(Object o, String s)
   {
      if (debug)
         print(o.getClass(), s);
   }

   public static void print(Class c, String s)
   {
      if (debug && debugging(c))
         debugout.println("["+ (null == c ? "null" : c.getName()) + "] " + s);
   }

   public static void printMap(Object o, Map m)
   {
      if (debug)
         printMap(o.getClass(), m);
   }
   
   public static void printMap(Class c, Map m)
   {
      if (debug && debugging(c)) {
         Iterator i = m.keySet().iterator();
         debugout.println("["+ (null == c ? "null" : c.getName()) +"] Map : ");
         while (i.hasNext()) {
            Object key = i.next();
            debugout.println("\t\t- "+key+" => "+m.get(key));
         }
      }
   }
}
