package org.ucam.srcf.assassins;

/**
 * Create news pages from event-list
 * @author Mike Cripps <mac57@srcf.ucam.org>
 */

import java.io.IOException;
import java.io.OutputStream;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

//import org.w3c.dom.*;

public class NewsCreator {

   Map dates;
   //Map pages;
   UpdatesCreator uc;
   HeadlinesCreator hc;
   private Map plugins;
   private Set pages;
   
   public NewsCreator(String path, Map p) throws ParseException, IOException
   {
      this(path, p, null);
   }
   public NewsCreator(String path, Map p, OutputStream errors) throws ParseException, IOException
   {
      plugins = p;
      dates = new TreeMap();
      String firstPageBaseName = Config.getConfig("vanilla","news-page-name");
      NewsPage firstPage = new NewsPage(path+firstPageBaseName+"00.xml", plugins);
      firstPage.setErrorStream(errors);
      if (Debug.debug) Debug.print(this, firstPage+".setLinkName("+firstPageBaseName+"00.html);");
      firstPage.setLinkName(firstPageBaseName+"00.html");
      dates.put(new Long(0), firstPage);
      if (Debug.debug) Debug.print(NewsCreator.class, "NewsCreator()");
      int i=1;
      boolean valid=true;
      while (valid) {
         if (Debug.debug) Debug.print(NewsCreator.class, "getting date "+i);
         String date = Config.getConfig("vanilla","news-date-"+i);
         if (Debug.debug) Debug.print(NewsCreator.class, "ok: >"+date+"<");
         valid = (null != date);
         if (valid) {
            if (Debug.debug) Debug.print(NewsCreator.class, "was valid");
            String pageName = Config.getConfig("vanilla","news-page-name")+((i<10?"0":"")+i);
            String k = path+pageName+".xml"; 
            long uft = Tools.unFormatTime(date);
            if (Debug.debug) Debug.print(NewsCreator.class, "uft: "+uft);
            //dates.put(new Long(uft),k);
            NewsPage newPage = new NewsPage(k, plugins);
            newPage.setErrorStream(errors);
            if (Debug.debug) Debug.print(this, newPage+".setLinkName("+pageName+".html);");
            newPage.setLinkName(pageName+".html");
            dates.put(new Long(uft), newPage);
       }
         if (Debug.debug) Debug.print(NewsCreator.class, "end of loop");
         i++;
      }
      String update = Config.getConfig("vanilla","updates-page-name");
      String headli = Config.getConfig("vanilla","headlines-page-name");
      uc = new UpdatesCreator(path+update+".xml", plugins);
      uc.setErrorStream(errors);
      hc = new HeadlinesCreator(path+headli+".xml", plugins);
      hc.setErrorStream(errors);
   }

   public void addEvent(Event e, Map m) throws AssassinSQLException {
      if (Debug.debug) Debug.print(NewsCreator.class, "Event add: "+e);
      Iterator z = dates.keySet().iterator();
      long t = e.getEventTime();
      long old = 0;
      while (z.hasNext()) {
         Long u = (Long) z.next();
         if (Debug.debug) Debug.print(NewsCreator.class, "Checking "+u.longValue());
         if (u.longValue()<=t) old=u.longValue();
      }
      NewsPage page =  (NewsPage) (dates.get(new Long(old)));
      page.addEvent(e,m,page);
      uc.addEvent(e,m,page);
      hc.addEvent(e,m,page);
      if (Debug.debug) Debug.print(NewsCreator.class, "NP01: event " + e.getEid() + " in page " + page.getLinkName());
      if (Debug.debug) Debug.print(NewsCreator.class, "Added to: "+old);
   }
   
   public void generate() throws AssassinSQLException, StateBuilderException {
      if (Debug.debug) Debug.print(NewsCreator.class, "Starting Generation");
      Collection c = dates.values();
      Iterator i = c.iterator();
      if (Debug.debug) Debug.print(NewsCreator.class, "iterating over pages");
      while (i.hasNext())
      {
         if (((NewsPage)i.next()).isEmpty())
         {
            if (Debug.debug) Debug.print(NewsCreator.class, "killing a blank page");
            i.remove(); // kill blank pages
         }
      }
      if (Debug.debug) Debug.print(NewsCreator.class, "done iterating");
      uc.setAllNewsPages(c);
      hc.setAllNewsPages(c);
      Iterator z = (c).iterator();
      while (z.hasNext()) {
      	NewsPage np = ((NewsPage)z.next());
         if (Debug.debug) Debug.print(NewsCreator.class, "Looping over page...");
         np.setAllNewsPages(c);
         this.pages=np.getAllNewsPages();
         if (Debug.debug) Debug.print(NewsCreator.class, "Calling np.generate()");
         np.generate();
      }
      if (Debug.debug) Debug.print(NewsCreator.class, "Doing updates and headlines");      
      uc.generate();
      hc.generate();
   }
   
   public Set getPages() {
   	return pages;
   }
}
