/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.NoDatabaseAssociativityException;
import org.ucam.srcf.assassins.exceptions.ObjectModifiedException;

/**
 * Class representing state changes for the addition of a new pseudonym to a
 * player.
 *
 * @author Matthew Johnson <autoumpire@matthew.ath.cx>
 */
public class PseudonymGain extends StateChange
{
   public PseudonymGain(Assassin a, Pseudonym p)
   {
      this.ass = a;
      this.pluginName = "vanilla";
      this.field = "pseudonym-gain";
      this.value = p;
   }

   PseudonymGain(int scid)
   {
      super(scid);
   }

   PseudonymGain(Integer scid)
   {
      super(scid);
   }

   /**
    * Retrieve the pseudonym being added.
    * @return A String representing the new pseudonym.
    */
   public String getPseudonym() throws ObjectModifiedException, NoDatabaseAssociativityException, AssassinSQLException
   {
      if (null == value)
         readDetails();
      return ((Pseudonym) value).getValue();
   }
   public String toString() { return "__::"+StateChange.class.getName()+"("+id+")"; }
}
