/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.NoDatabaseAssociativityException;
import org.ucam.srcf.assassins.exceptions.ObjectModifiedException;

/**
 * Class representing the reports which can be attached to events.
 *
 * @author Matthew Johnson <autoumpire@matthew.ath.cx>
 */
public class Report extends CanonicalEntity
{
   private static Map reportCache = new HashMap();
   private static Vector newReportCache = new Vector();
   private static boolean preCacheState = false;
   
   /**
    * Loads all the objects into memory frm the database in one query
    */
   static void preCache() throws AssassinSQLException
   {
      if (preCacheState) return;
      preCacheState = true;
      AResultSet r = DatabaseBackend.query("SELECT rid, event, assassin, pseudonym, text FROM "+DatabaseBackend.getPrefix()+"_reports");
      try {
         while (r.encapsulated.next()) {

            int rid = r.encapsulated.getInt("rid");
            Integer RID = new Integer(rid);

            // check the cache and add as necessary
            if (!reportCache.containsKey(RID)) {
               Report re = new Report(rid);
               re.event = Event.getByID(r.encapsulated.getInt("event"));
               re.assassin = Assassin.getByAid(r.encapsulated.getInt("assassin"));
               re.pseudonym = Pseudonym.getByPsid(r.encapsulated.getInt("pseudonym"));
               re.text = r.encapsulated.getString("text");
               re.read = true;
               reportCache.put(RID, re);
            }
         }
         r.close();
      } catch (SQLException sqle) {
         throw new AssassinSQLException();
      }
   }

   private static List getReportsFromQuery(String query) throws AssassinSQLException
   {
      List l = new Vector();
      AResultSet r = DatabaseBackend.query(query);
      try {
         while (r.encapsulated.next()) {

            int rid = r.encapsulated.getInt("rid");
            Integer RID = new Integer(rid);

            // check the cache and add as necessary
            if (!reportCache.containsKey(RID)) {
               Report re = new Report(rid);
               re.event = Event.getByID(r.encapsulated.getInt("event"));
               re.assassin = Assassin.getByAid(r.encapsulated.getInt("assassin"));
               re.pseudonym = Pseudonym.getByPsid(r.encapsulated.getInt("pseudonym"));
               re.text = r.encapsulated.getString("text");
               re.read = true;
               reportCache.put(RID, re);
               l.add(re);
            }
            else
               l.add(reportCache.get(RID));
         }
         r.close();
      } catch (SQLException sqle) {
         throw new AssassinSQLException();
      }
      return l;
   }
  
   static List getReportsBy(Player p) throws AssassinSQLException
   {
      return getReportsFromQuery("SELECT rid, event, assassin, pseudonym, text FROM "+DatabaseBackend.getPrefix()+"_reports INNER JOIN "+DatabaseBackend.getPrefix()+"_assassins ON assassin=aid WHERE pid = '"+p.getPid()+"'");
   }
 
   static List getReportsBy(Assassin a) throws AssassinSQLException
   {
      return getReportsFromQuery("SELECT rid, event, assassin, pseudonym, text FROM "+DatabaseBackend.getPrefix()+"_reports WHERE assassin = '"+a.getAid()+"'");
   }
 
   static List getAllReports() throws AssassinSQLException
   {
      return getReportsFromQuery("SELECT rid, event, assassin, pseudonym, text FROM "+DatabaseBackend.getPrefix()+"_reports");
   }
   

   /**
    * Returns a Report with a specific database ID
    *
    * @param rid The Database ID
    * @return The Report object
    */  
   static Report getByRid(int rid) throws AssassinSQLException
   {
      if (0 == rid) return null;
      Report r;

      // check cache
      if (null == (r = (Report) reportCache.get(new Integer(rid)))) {
         // check database existence
         // String query = "SELECT rid FROM " + DatabaseBackend.getPrefix() + "_reports WHERE rid = '"+rid+"'";
         // ResultSet rs;
         // try {
         //    rs = DatabaseBackend.query(query);
         //    if (!rs.next()) throw new NonExistentEntityException("Eek");
         //    rs.close();
         // } catch (SQLException sqle) {
         //    throw new AssassinSQLException(sqle);
         // }
         // add to cache
         r = new Report(rid);
         reportCache.put(new Integer(rid), r);
      }
      return r;
   }
      
   /**
    * Returns a Report with a specific database ID
    *
    * @param id The Database ID
    * @return The Report object
    */
   static Object getByID(Integer id) throws AssassinSQLException
   {
      return getByRid(id.intValue());
   }
 
   /**
    * Commit all changes to Reports to the database
    */
   static void commitAll() throws AssassinSQLException
   {
      // iterate over all new Report objects
      while (newReportCache.size() > 0) {
         Report a = (Report) newReportCache.remove(0);
         a.writeDetails();
         reportCache.put(new Integer(a.getRid()),a);
      }
      
      // iterate over all cached Report objects
      Iterator i = reportCache.values().iterator();
      while (i.hasNext()) {
         Report r = (Report) i.next();
         // if they are modified, then update
         if (r.dirty()) r.writeDetails();
      }
   }
      
   /** The event that this report is associated with */
   private Event event = null;
   private Assassin assassin = null;
   private Pseudonym pseudonym = null;
   private String text = null;
   private boolean read = false;

   public Report()
   {
      this.id = 0;
      this.text = "";
      this.read = true;
      newReportCache.add(this);
   }
   
   Report(int rid)
   {
      this.id = rid;
      clean();
   }

   Report(Integer rid)
   {
      this(rid.intValue());
   }
   
   /**
    * Retrieve the assassin making this report.
    * @return The object representing the assassin.
    */
   Assassin getAssassin() throws AssassinSQLException 
   {
      if (!read) readDetails();
      return assassin;
   }
   
   /**
    * Retrieve the assassin making this report.
    * @return The object representing the assassin.
    */
   public AssassinViewer getAssassinViewer() throws AssassinSQLException
   {
      if (!read) readDetails();
      if (null == assassin) return null;
      return new AssassinViewer(assassin);
   }
   
   /**
    * Set which assassin this report is attributed to.
    * @param a The assassin to attribute the report to.
    */
   public void setAssassin(Assassin a)
   {
      setDirty();
      this.assassin = a;
   }

   /**
    * Retrieve which pseudonym is to be used. 
    * @return The pseudonym object.
    */
   public Pseudonym getPseudonym() throws AssassinSQLException
   {
      if (!read) readDetails();
      return pseudonym;
   }
   
   /**
    * Set the number of the pseudonym to use for this report.
    * @param pidx The number of the pseudonym to use.
    * @see #getPseudonym()
    */
   public void setPseudonym(Pseudonym p)
   {
      setDirty();
      this.pseudonym = p;
   }

   /**
    * Retrieve which event this report is to be associated with.
    * @return The object representing the event the report is linked to.
    */
   public Event getEvent() throws AssassinSQLException
   { 
      if (!read) readDetails();
      return event; 
   }
   
   /**
    * Set which event this report is attached to.
    * @param e The event to attach this report to.
    */
   void setEvent(Event e) 
   { 
      setDirty();
      this.event = e; 
   }
   
   /**
    * Retrieve the text of the report.
    * @return The report text.
    */
   public String getText() throws AssassinSQLException
   {
      if (!read) readDetails();
      return text;
   }
   
   /**
    * Set the text of this report.
    * @param text The new report text.
    */
   public void setText(String text)
   {
      setDirty();
      this.text = text;
   }

   public int getRid() {
      return this.id;
   }

   /**
    * Update the details of this from the database
    */
   void readDetails() throws AssassinSQLException
   {
      if (dirty()) throw new ObjectModifiedException();

      if (0 == id) throw new NoDatabaseAssociativityException();

      read = true;

      String query = "SELECT event, assassin, pseudonym, text FROM " + DatabaseBackend.getPrefix() + "_reports WHERE rid = '"+id+"'";
      AResultSet r;
      
      try {
         r = DatabaseBackend.query(query);
         if (! r.encapsulated.next()) throw new NoDatabaseAssociativityException();
         this.assassin = Assassin.getByAid(r.encapsulated.getInt("assassin"));
         this.event = Event.getByID(r.encapsulated.getInt("event"));
         this.pseudonym = Pseudonym.getByPsid(r.encapsulated.getInt("pseudonym"));
         this.text = r.encapsulated.getString("text");
         r.close();
      } catch (SQLException sqle) {
         throw new AssassinSQLException(sqle);
      }
   }

   /**
    * Update the details of this int the database
    */
   void writeDetails() throws AssassinSQLException
   {
      // cases:
      //  - rid == 0, event == null: ignore
      //  - rid == 0, event != null: add
      //  - rid != 0, event == null: delete
      //  - rid != 0, event != null: update
      
      if (!dirty()) return;
      if (Debug.debug) Debug.print(this, "Writing details");
      clean();      
      if (newReportCache.contains(this)) newReportCache.remove(this);

      // ignore
      if (0 == id && null == event) return;
      
      // insert new
      if (0 == id && null != event) {
         event.writeDetails();
         if (null != pseudonym) pseudonym.writeDetails();
         if (null != assassin) assassin.writeDetails();
         String f =  "event, text";
         String v = "'"+event.getEid()+"','"+Tools.escape(text)+"'";
         if (null != pseudonym) {
            f += ",pseudonym";
            v += ",'"+pseudonym.getPsid()+"'";
         }
         if (null != assassin) {
            f += ",assassin";
            v += ",'"+assassin.getAid()+"'";
         }
         String query = "INSERT INTO " + DatabaseBackend.getPrefix() + "_reports ("+f+") VALUES ("+v+")";
         try{
			 DatabaseBackend.query(query).close();
         } catch (SQLException e) {
			 throw new AssassinSQLException(e);
         }
         query = "SELECT LAST_INSERT_ID() FROM " + DatabaseBackend.getPrefix() + "_reports";
         AResultSet r;
         try {
            r = DatabaseBackend.query(query);
            if (!r.encapsulated.next()) throw new AssassinSQLException("Oh Ek! That shouldn't happen?!");
            id = r.encapsulated.getInt(1);
            r.close();
         } catch (SQLException sqle) {
            throw new AssassinSQLException(sqle);
         }
      }

      // delete
      if (0 != id && null == event) {
         String query = "DELETE FROM " + DatabaseBackend.getPrefix() + "_reports WHERE rid = '"+id+"'";
         try{
			 DatabaseBackend.query(query).close();
         } catch (SQLException e) {
			 throw new AssassinSQLException(e);
         }
         //reportCache.remove(new Integer(rid));
         id = 0;
      }
      
      // update existing
      if (0 != id && null != event) {
         if (null != this.event) event.writeDetails();
         if (null != this.pseudonym) pseudonym.writeDetails();
         if (null != this.assassin) assassin.writeDetails();
         String query = "UPDATE " + DatabaseBackend.getPrefix() + "_reports SET ";
         if (null != this.text) query += "text = '" + Tools.escape(this.text) + "', ";
         if (null != this.event) query += "event = '" + this.event.getEid() + "', ";
         if (null != this.assassin) query += "assassin = '" + this.assassin.getAid() + "', ";
         if (null != this.pseudonym) query += "pseudonym = '" + this.pseudonym.getPsid() + "', ";
         else query += "pseudonym = NULL, ";
         query += " rid = '"+id+"' WHERE rid = '"+id+"'";
         try{
			 DatabaseBackend.query(query).close();
         } catch (SQLException e) {
			 throw new AssassinSQLException(e);
         }
      }
   }

   protected void setDirty()
   {
      try {
         if (!this.read) this.readDetails();
      } catch (AssassinSQLException ASQLe) {}
      super.setDirty();
      if (null != event)
         event.setModifiedTime(System.currentTimeMillis());
   }

   public Viewer getViewer()
   {
      return null;
   }
   
   public String getPrintable() throws AssassinSQLException
   {
      if (!read) readDetails();
      if (null != assassin)
         return "Report by <span class=\"playername\">"+getAssassin().getPlayer().getName()+"</span> on <span class=\"reportheadline\">"+getEvent().getReplaceHeadline()+"</span>";
      return "Comment on <span class=\"reportheadline\">"+getEvent().getReplaceHeadline()+"</span>";
   }

}

