/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.HashMap;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * This just provides a compatible interface to the Plugins for liveness and activeness.
 */
public class Vanilla extends Plugin
{

	public String getDescription() {
		return "Base AutoUmpire plugin";
	}

   /**
    * Called by the filtering mechanism when the plugin needs to determine
    * whether an assassin has a given property such as &quot;police&quot;. If
    * the plugin does not recognise the named property, it should return
    * <code>false</code>.
    * @param a A viewer for the assassin being tested.
    * @param prop The name of the property being asked about.
    * @return True if the assassin has the property, false if not.
    */
   public boolean assassinHasProperty(AssassinStateViewer a, String prop, String value)  throws AssassinSQLException
   {
      if (Debug.debug) Debug.print(this, "assassinHasProperty");
      if (Debug.debug) Debug.print(this, "Parameters: ("+a+", "+prop+","+value+")");
   
      if (prop.equals("live")) {
         if (value.toLowerCase().equals("true"))
            return a.isAlive();
         return ! a.isAlive();
      } else if (prop.equals("active")) {
         if (value.toLowerCase().equals("true"))
            return a.isActive();
         return ! a.isActive();
      } else
         return false;
   }

   /**
    * Returns a Map of new commands to add to the CLI
    *
    * @return A Map of String =&gt; Command
    */
   public Map getCommands() { return new HashMap(); }

}


