/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.peach;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.AuthLevel;
import org.ucam.srcf.assassins.Authentication;
import org.ucam.srcf.assassins.CanonicalEntity;
import org.ucam.srcf.assassins.CanonicalEntityField;
import org.ucam.srcf.assassins.CanonicalViewer;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.Event;
import org.ucam.srcf.assassins.HTMLForm;
import org.ucam.srcf.assassins.Player;
import org.ucam.srcf.assassins.PlayerViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.PluginCGICommand;
import org.ucam.srcf.assassins.PluginCommand;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.Password;
import cx.ath.matthew.cgi.SubmitButton;

/**
 * Peach manages the passwords for the CGI interface
 */
public class Peach extends Plugin
{
   public String getDescription() { return "CGI password management."; };
   private Random rnd=null;
   private char pwd_chars[]=null;
   
   /**
    */
   public Peach()
   {
      super();

      /*
      * Sets up chars in the pwd_chars array.
      * a-z, A-Z, 0-9,
      */
      pwd_chars=new char[62];
      int i=0;
      for(char c='a';c<='z';c++)
         pwd_chars[i++]=c;
      for(char c='A';c<='Z';c++)
         pwd_chars[i++]=c;
      for(char c='0';c<='9';c++)
         pwd_chars[i++]=c;
      //pwd_chars[i++]=',';
      //pwd_chars[i++]='.';

      /*Initialize our random number gen*/
      rnd=new Random(System.currentTimeMillis());
   }
   
   /**
    * Called during the updating of the gamestate to ask whether a given player
    * requires email to be sent to them during the current round of emails.
    * @param p A viewer for the player being asked about.
    * @return True if the plugin requires email to be sent to the player in
    *    question, false if not.
    */
    protected boolean playerNeedsEmail(AssassinStateViewer asv) throws AssassinSQLException
   {
      boolean haspw = asv.getBoolProperty(this, "haspw");
      if (Debug.debug) Debug.print(this, "Does "+asv.getAssassin().getPlayerViewer().getName()+" need an email? answer: "+(!haspw));
      if (!haspw) return true;
      return false;
   }

   /**
    * Called when the system has decided that email needs to be sent to a given
    * player. The plugin may return a string to be included in the email, or
    * null if it does not wish to include anything.
    * @param p A viewer for the player being emailed.
    * @return A string to be included in the email sent to the player, or null.
    *    The string will appear in its own paragraph when inserted into the
    *    email and so should not begin or end with a newline.
    */
    protected String generateEmail(AssassinStateViewer asv, Map state) throws AssassinSQLException
   {
      String password = asv.getAssassin().getPlayerViewer().getPassword();
      boolean haspw = asv.getBoolProperty(this, "haspw");
      String message = "Your password for logging in to the Auto Umpire is `"+password+"'. Be careful that this password is case-sensitive.";
      if (!haspw) {
         if (Debug.debug) Debug.print(this, "Marking "+asv.getAssassin().getPlayerViewer().getName()+" as had an email");
         // add a state change setting haspw
         Event e = new Event();
         e.setEventTime(System.currentTimeMillis());
         e.setVisible(false);
         e.setActive(true);
         e.addStateChange(new PluginStateChange(asv.getAssassin(),this, "haspw", "true" ));
         message = "Your NEW password for logging in to the Auto Umpire is `"+password+"'. Be careful that this password is case-sensitive.";
      }

      if (null != password && ! password.equals("")) return message;
      return "";
   }

   private AssassinViewer findAssassin(String email) throws AssassinSQLException, StateBuilderException
   {
      // FIXME!! Must be a better way of doing this
      Map state = vanilla.getState(Assassin.getAllAssassinViewersWithEmail(email));
      Iterator i = state.keySet().iterator();
      while (i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         if (!((AssassinStateViewer) state.get(a)).isAlive()) continue;
         if (!((AssassinStateViewer) state.get(a)).isActive()) continue;
         if (a.getPlayerViewer().getEmail().equals(email)) return a;
      }
      while (i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         if (a.getPlayerViewer().getEmail().equals(email)) return a;
      }
      return null; // this shouldn't happen
   }
   
   private AssassinViewer findAssassin(PlayerViewer player) throws AssassinSQLException, StateBuilderException
   {
      return findAssassin(player.getEmail());
   }

   protected void changepw(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException
   {
      cgi.out("<h3>Change Your Password</h3>");
      // show password form
      if (params.get("submit") == null) {
         HTMLForm form = new HTMLForm("umpire.cgi?change-password", "Change");
         form.addField(new Password("newpw", "Enter Your New Password:", ""));
         form.addField(new Password("newpw2", "Re-enter Password:", ""));
         form.display(cgi);
      }

      // Set Password
      else {
         String newpw = (String) params.get("newpw");
         String newpw2 = (String) params.get("newpw2");
         cgi.out("<p>");
         if (!"".equals(newpw) && newpw != null && newpw.equals(newpw2)) {
            if (vanilla.updateAuthenticationDetails(Authentication.currentAuth.getEmail(), newpw)) {
               // add a state change setting haspw to false
               if (Debug.debug) Debug.print(this, "Marking "+Authentication.currentAuth.getEmail()+" as needing an email");
               AssassinViewer a = findAssassin(Authentication.currentAuth.getEmail());
               markForMail(a);
               cgi.out("Password change succeeded. You will have to log in with the new password on the next page.");
            }
            else
               cgi.out("Password change failed.");
         }
         else 
            cgi.out("Password Left Blank or Passwords Differ! Please go back and try again.");
         cgi.out("</p>");
      }
   }

   protected void setpw(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException
   {
      cgi.out("<h3>Set A Password</h3>");
      // show password form
		if(Authentication.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
         if (params.get("submit") == null) {
            HTMLForm form = new HTMLForm("umpire.cgi?set-password", "Set");
            form.addField(new CanonicalEntityField("newuser", "Select Player to Set Password:", 
                     (CanonicalViewer[]) Player.getAllPlayerViewers().toArray(new CanonicalViewer[] {}), 
                     (CanonicalEntity) null));
            form.addField(new Password("newpw", "Enter New Password:", ""));
            form.addField(new Password("newpw2", "Re-enter Password:", ""));
            form.addField(new SubmitButton("submit", "Random Password"));
            form.display(cgi);
         }

         // Set Password
         else {
            String newpw;
            String newpw2;
            if ("Random Password".equals(params.get("submit")))
               newpw2 = newpw = getRandPass(8);
            else {
               newpw = (String) params.get("newpw");
               newpw2 = (String) params.get("newpw2");
            }
            PlayerViewer player = (PlayerViewer) params.get("newuser");
            cgi.out("<p>");
            if (!"".equals(newpw) && newpw != null && newpw.equals(newpw2)) {
               if (vanilla.updateAuthenticationDetails(player.getEmail(), newpw)) {
                  // add a state change setting haspw to false
                  if (Debug.debug) Debug.print(this, "Marking "+player.getName()+" as needing an email");
                  AssassinViewer a = findAssassin(player);
                  markForMail(a);
                  cgi.out("<div class=\"donemessage\">");
                  cgi.out("<p><b>Password change succeeded.</b></p>");
				  cgi.out("<p>&nbsp,</p>");
				  cgi.out("</div>");
               }
               else {
                  cgi.out("<div class=\"warningmessage\">");
                  cgi.out("<p><b>Password change failed.</b></p>");
				  cgi.out("<p>&nbsp;</p>");
			   }
            }
            else 
                cgi.out("<div class=\"warningmessage\">");
				cgi.out("<p><b>Password change failed.</b></p>");
				cgi.out("<p>Password Left Blank or Passwords Differ! Please go back and try again.</p>");
				cgi.out("</div>");
         }
      }

      else {
          cgi.out("<div class=\"errormessage\">");
         cgi.out("<p><b>Insufficent Privileges</b></p>");
         cgi.out("<p>You must be the Umpire to set another user's password.</p>");
		 cgi.out("</div>");
      }
   }

	private void markForMail(AssassinViewer a) throws AssassinSQLException
	{
      if (a != null) {
         Event e = new Event();
         e.setEventTime(System.currentTimeMillis());
         e.setVisible(false);
         e.setActive(true);
         e.addStateChange(new PluginStateChange(a,this, "haspw", "false" ));
      }
	}

   protected void issuepws(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException, IOException
   {
      cgi.out("<h3>Issue Passwords</h3>");
		if (Authentication.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
         cgi.out("<p>");
         cgi.out("   Issuing passwords....");
         cgi.out("</p>");
         cgi.flush();
         List ps = Player.getAllPlayerViewers();
         Iterator i = ps.iterator();
         while (i.hasNext()) {
            PlayerViewer p = (PlayerViewer) i.next();
            if (null == p.getPassword() || "".equals(p.getPassword())) {
               vanilla.updateAuthenticationDetails(p.getEmail(), getRandPass(8));
               markForMail(findAssassin(p));
            }
         }
         cgi.out("<p>");
         cgi.out("   All players without passwords have been issued new passwords.");
         cgi.out("</p>");
      }
      else {
         cgi.out("<p>");
         cgi.out("   You must be the Umpire to issue passwords.");
         cgi.out("</p>");
      }
   }

   private String getRandPass(int length)
   {
      String password=new String("");
      for(int i=0;i<length;i++)
         password+=pwd_chars[rnd.nextInt(pwd_chars.length)];
      return password;
   }

   protected void setpw(Map params) throws AssassinSQLException, StateBuilderException
   {
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      String user = (String) params.get("-u");
      String password = (String) params.get("-p");
      try {
         if (null == user) {
            System.out.print("Enter email address to set password for: ");
            user = in.readLine();
         }
         System.out.println("Setting password for "+user);
         if (null == password) {
            System.out.print("Enter new password: ");
            password = in.readLine();
         }
      } catch (IOException IOe) { 
         System.out.println("Password change falied. Could not read user or password.");
         return;
      }
      if (vanilla.updateAuthenticationDetails(user, password)) {
         // add a state change setting haspw to false
         if (Debug.debug) Debug.print(this, "Marking "+user+" as needing an email");
         markForMail(findAssassin(user));
         System.out.println("Password change succeeded.");
      } else
         System.out.println("Password change falied. Is "+user+" a valid email address?");
   }

   protected void issuepws(Map params) throws AssassinSQLException, StateBuilderException
   {
      List ps = Player.getAllPlayerViewers();
      Iterator i = ps.iterator();
      while (i.hasNext()) {
         PlayerViewer p = (PlayerViewer) i.next();
         if (null == p.getPassword() || "".equals(p.getPassword())) {
            vanilla.updateAuthenticationDetails(p.getEmail(), getRandPass(9));
            markForMail(findAssassin(p));
         }
      }
      System.out.println("All players with unset passwords have been issued passwords.");
   }

   /**
    * Returns a Map of new commands to add to the CGI
    *
    * @return A Map of String =&gt; CGICommand
    */
   public Map getCGICommands()
   { 
      Map m = new HashMap();
      final Peach peach = this;
      m.put("change-password",
            new PluginCGICommand () {
               public void pluginCommand(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException
               {
                  peach.changepw(cgi, params);
               }  
               public String getName() { return "Change Password"; }
               public String getDescription() { return "Change your password"; }
               public AuthLevel accessLevel() { return AuthLevel.LOGGED_IN; }
			   public boolean isVisible() {	return true; }
            }
           );
      m.put("issue-passwords",
            new PluginCGICommand () {
               public void pluginCommand(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException, IOException
               {
                  peach.issuepws(cgi, params);
               }  
               public String getName() { return "Issue Passwords"; }
               public String getDescription() { return "Create random passwords for all players"; }
               public AuthLevel accessLevel() { return AuthLevel.UMPIRE; }
			   public boolean isVisible() {	return true; }
            }
           );
      m.put("set-password",
            new PluginCGICommand () {
               public void pluginCommand(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException, CGIInvalidContentFormatException
               {
                  peach.setpw(cgi, params);
               }  
               public String getName() { return "Set Password"; }
               public String getDescription() { return "Set the password for a player"; }
               public AuthLevel accessLevel() { return AuthLevel.UMPIRE; }
			   public boolean isVisible() {	return true; }
            }
           );
      return m;
   }


   /**
    * Returns a Map of new commands to add to the CLI
    *
    * @return A Map of String =&gt; Command
    */
   public Map getCommands()
   { 
      Map m = new HashMap();
      final Peach peach = this;
      m.put("issue-passwords",
            new PluginCommand () {
               public void pluginCommand(Map params) throws AssassinSQLException, StateBuilderException
               {
                  peach.issuepws(params);
               }  
            }
           );
      m.put("set-password",
            new PluginCommand () {
               public void pluginCommand(Map params) throws AssassinSQLException, StateBuilderException
               {
                  peach.setpw(params);
               }  
            }
           );
      return m;
   }

}


