/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.stracciatella;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.Resurrection;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.YesNoQuestion;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

class Node
{
   private static HashMap nodeTable = new HashMap();
   private static Vector liveNodes = new Vector();
   public static Node findNode(AssassinViewer assassin)
   {
      return (Node) nodeTable.get(assassin);
   }
   public static Iterator iterator() { return nodeTable.values().iterator(); }
   public static ListIterator liveIterator() { return liveNodes.listIterator(); }
   public static int number() { return nodeTable.size(); }
   private AssassinViewer assassin;
   private int oldkills = 0;
   private int oldconkers = 0;
   private int oldrank = 0;
   private int olddeaths = 0;
   private int oldkilldeaths = 0;
   private int newkills = 0;
   private int newconkers = 0;
   private int newrank = 1;
   private int newdeaths = 0;
   private int newkilldeaths = 0;
   private boolean live = true;
   private boolean mark = false;
   private boolean modified = false;
   private Vector killv = new Vector();
   public Node(AssassinViewer assassin) 
   { 
      this.assassin = assassin;
      nodeTable.put(assassin, this);
      liveNodes.add(this);
   }
   public Node(AssassinViewer assassin, int rank, int kills, int conkers, int deaths, int killdeaths) 
   { 
      this.assassin = assassin;
      this.oldrank = rank;
      this.oldkills = kills;
      this.oldconkers = conkers;
      this.olddeaths = deaths;
      this.oldkilldeaths = killdeaths;
      nodeTable.put(assassin, this);
      liveNodes.add(this);
   }
   public void addKill(Node n) 
   {
      killv.add(n); 
      n.newdeaths++;
      if (live) {
         n.live = false;
         if (liveNodes.contains(this))
            liveNodes.remove(n);
      }
   }
   public boolean isLive() { return live; }
   public void resurrect() { live = true; }
   public void kill() { live = false; }
   public boolean isModified() { return modified; }
   public int getKills() { return newkills; }
   public int getDeaths() { return newdeaths; }
   public int getRank() { return newrank; }
   public void setRank(int rank) { this.newrank = rank; }
   public int getConkers() { return newconkers; }
   public int getKillDeaths() { return newkilldeaths; }
   public AssassinViewer getAssassin() { return assassin; }
   public void calculate()
   {
      if (mark) return;
      mark = true;
      Iterator i = killv.iterator();
      while (i.hasNext()) {
         newkills++;
         Node n = (Node) i.next();
         if (!n.mark) n.calculate();
         newconkers += n.getConkers() + 1;
      }
      newkilldeaths = newkills-newdeaths;
      if (oldkills != newkills || oldconkers != newconkers || oldrank != newrank || olddeaths != newdeaths || oldkilldeaths != newkilldeaths) {
         modified = true;
         if (Debug.debug) try { Debug.print(Stracciatella.class,
               "Calculated "+assassin.getPlayerViewer().getName()+
               " kills "+newkills+" conkers "+newconkers+" rank "+newrank+" deaths "+newdeaths+" kills-deaths "+newkilldeaths);
         } catch (Exception e) {}
      }
   }
}


/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public class Stracciatella extends Plugin
{
   public String getDescription() { return "Stats building module (kills, conkers, position)"; };
   /**
    * Called by the filtering mechanism when the plugin needs to determine
    * whether an assassin has a given property such as &quot;police&quot;. If
    * the plugin does not recognise the named property, it should return
    * <code>false</code>.
    * @param a A viewer for the assassin being tested.
    * @param prop The name of the property being asked about.
    * @return True if the assassin has the property, false if not.
    */
   public boolean assassinHasProperty(AssassinStateViewer a, String prop, String value)  throws AssassinSQLException
   {
      if ("hasstats".equals(prop)) {
         String v = a.getProperty(getPluginName(), "hasstats");
         return value.equals(v);
      }
      else return false;
   }

   /**
    * Called when an assassin is being added to get any queries the plugin
    * has for the user. The plugin should check all the assassin details and if it
    * needs more information it should ask relevent questions. The 
    * returnQuestions function is passed the same Vector with the answers 
    * filled out.
    * @param a The Assassin which is being added
    * @return A Vector of Questions
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @see returnQuestions
    */
   protected Vector assassinAdded(AssassinViewer a, EventViewer e, Map state, Vector qs) throws AssassinSQLException 
   {
      if ( null == qs) {
         qs = new Vector();
         qs.add(new YesNoQuestion("Track Stats",true));
         return qs;
      } else {
         boolean dostats = ((YesNoQuestion)qs.get(0)).getAnswer();
         vanilla.addStateChange(e, new PluginStateChange(a, this, "hasstats", dostats?"true":"false"));
         return null;
      }
   }

   protected Vector stateChangeAdded(StateChange sc, Map state, Vector qs)  throws AssassinSQLException
   { 
      if (Debug.debug) Debug.print(this, "stateChangeAdded");
      if (null == qs) if (sc instanceof Death) {
         Death d = (Death) sc;
         EventViewer e = sc.getEventViewer();
         AssassinViewer kv = d.getKiller();
         AssassinViewer vv = d.getVictim();
         AssassinStateViewer ksv = (AssassinStateViewer) state.get(kv);
         AssassinStateViewer vsv;
         try { 
            vsv = vanilla.getState(vv, e.getTime()-1);
         } catch (StateBuilderException SBe) {
            vsv = (AssassinStateViewer) state.get(vv);
         }
         if (Debug.debug) Debug.print(this, "got "+kv+" killed "+vv+" with killer live: "+ksv.isAlive()+" and victim live: "+vsv.isAlive());
         if (/*ksv.isAlive() &&*/ vsv.isAlive()) {
            int kills = ksv.getIntProperty(this, "kills") + 1;
            if (Debug.debug) Debug.print(this, "new kills: "+kills);
            vanilla.addStateChange(sc.getEventViewer(), new PluginStateChange(kv, this, "kills", ""+kills));
            int killdeaths = ksv.getIntProperty(this, "killdeaths") + 1;
            if (Debug.debug) Debug.print(this, "new killdeaths: "+killdeaths);
            vanilla.addStateChange(sc.getEventViewer(), new PluginStateChange(kv, this, "killdeaths", ""+killdeaths));
            int deaths = vsv.getIntProperty(this, "deaths") + 1;
            if (Debug.debug) Debug.print(this, "new deaths: "+deaths);
            vanilla.addStateChange(e, new PluginStateChange(vv, this, "deaths", ""+deaths));
            killdeaths = ksv.getIntProperty(this, "killdeaths") - 1;
            if (Debug.debug) Debug.print(this, "new killdeaths: "+killdeaths);
            vanilla.addStateChange(sc.getEventViewer(), new PluginStateChange(vv, this, "killdeaths", ""+killdeaths));
         }
      }
      return new Vector(); 
   }

   protected Vector updateState(Map state, Vector qs, EventViewer e) throws AssassinSQLException, ParseException 
   {
      if (Debug.debug) Debug.print(this, "Updating Stats");
      Iterator si = state.values().iterator();
      while (si.hasNext()) {
         AssassinStateViewer asv = (AssassinStateViewer) si.next();
         if (asv.getBoolProperty(this, "hasstats")) {
            if (Debug.debug) Debug.print(this, asv.getAssassin().getPlayerViewer().getName()+"("+asv.getAssassin().getAid()+") has stats enabled ("+asv.getIntProperty(this, "rank")+","+asv.getIntProperty(this, "kills")+","+asv.getIntProperty(this, "conkers")+","+asv.getIntProperty(this, "deaths")+","+asv.getIntProperty(this, "killdeaths")+")");
            new Node(asv.getAssassin(), 
                     asv.getIntProperty(this, "rank"),
                     asv.getIntProperty(this, "kills"), 
                     asv.getIntProperty(this, "conkers"),
                     asv.getIntProperty(this, "deaths"),
                     asv.getIntProperty(this, "killdeaths"));
         }
      }

      if (Debug.debug) Debug.print(this, "Walking Death List");
      List scs = StateChange.getVitalStatusUntilTime(vanilla.getNowTime());
      ListIterator i = scs.listIterator();
      int crank = Node.number();
      int drawn = 0;
      long ctime = 0;
      while (i.hasNext()) {
         StateChange sc = null;
         try {
            sc = (StateChange) i.next();
         } catch (ArrayIndexOutOfBoundsException AIOOBe) { break; /*this should never happen, but apparrently it does */}
         
         if (sc instanceof Resurrection) {
            AssassinViewer av = sc.getAssassinViewer();
            Node n = Node.findNode(av);
            i.remove();
            if (null == n) continue;
            n.resurrect();
            continue;
         } else if (sc instanceof Death) {
            Death d = (Death) sc;
            if (d.getEventViewer().getTime() > ctime) {
               ctime = d.getEventViewer().getTime();
               crank -= drawn;
               if (drawn > 1) {
                  if (Debug.debug) Debug.print(this, "Resolving Ties");
                  try {
                     i.previous();
                     for (int j = 1; j <= drawn; j++) {
                        Death dp = (Death) i.previous();
                        AssassinViewer victim = dp.getVictim();
                        if (Debug.debug) Debug.print(this, "Fixing Rank "+victim.getPlayerViewer().getName()+" to "+(crank+1));
                        Node vnode = Node.findNode(victim);
                        if (null == vnode) continue;
                        vnode.setRank(crank+1);
                     }
                     for (int j = 0; j <= drawn; j++) i.next();
                  } catch (ArrayIndexOutOfBoundsException AIOOBe) { /*this should never happen, but apparrently it does */}
               }
               drawn = 0;
            }
            AssassinViewer killer = d.getKiller();
            AssassinViewer victim = d.getVictim();
            Node knode = Node.findNode(killer);
            Node vnode = Node.findNode(victim);
            if (Debug.debug) Debug.print(this, "Got death: "+killer.getPlayerViewer().getName()+"("+killer.getAid()+") killed "+victim.getPlayerViewer().getName()+"("+victim.getAid()+")");
            if (null == vnode) continue;
            if (vnode == knode) {
               vnode.setRank(crank);
               drawn++;
               vnode.kill();
               continue;
            }
            if (!vnode.isLive()) continue;
            if (null == knode) {
               vnode.setRank(crank);
               drawn++;
               vnode.kill();
               continue;
            }
            vnode.setRank(crank);
            drawn++;
            //if (!knode.isLive()) continue;
            knode.addKill(vnode);
         }
      }
      crank -= drawn;
      if (drawn > 1) {
         if (Debug.debug) Debug.print(this, "Resolving Ties");
         for (int j = 1; j <= drawn; j++) {
            try {
               Death dp = (Death) i.previous();
               AssassinViewer victim = dp.getVictim();
               if (Debug.debug) Debug.print(this, "Fixing Rank "+victim.getPlayerViewer().getName()+" to "+(crank+1));
               Node vnode = Node.findNode(victim);
               if (null == vnode) continue;
               vnode.setRank(crank+1);
            } catch (ArrayIndexOutOfBoundsException AIOOBe) { /*this should never happen, but apparrently it does */}
         }
      }
      i = Node.liveIterator();
      while (i.hasNext()) {
         Node n = (Node) i.next();
         if (Debug.debug) Debug.print(this, n.getAssassin().getPlayerViewer().getName()+" alive, building their tree");
         n.calculate();
      }
      if (Debug.debug) Debug.print(this, "Walking Node List");
      Iterator ni = Node.iterator();
      while (ni.hasNext()) {
         Node n = (Node) ni.next();
         if (n.isModified()) {
            AssassinViewer a = n.getAssassin();
            if (Debug.debug) Debug.print(this, "adding statechange ("+a+",this,kills,"+n.getKills()+")");
            vanilla.addStateChange(e, new PluginStateChange(a, this, "kills", ""+n.getKills()));
            if (Debug.debug) Debug.print(this, "adding statechange ("+a+",this,conkers,"+n.getConkers()+")");
            vanilla.addStateChange(e, new PluginStateChange(a, this, "conkers", ""+n.getConkers()));
            if (Debug.debug) Debug.print(this, "adding statechange ("+a+",this,rank,"+n.getRank()+")");
            vanilla.addStateChange(e, new PluginStateChange(a, this, "rank", ""+n.getRank()));
            if (Debug.debug) Debug.print(this, "adding statechange ("+a+",this,deaths,"+n.getDeaths()+")");
            vanilla.addStateChange(e, new PluginStateChange(a, this, "deaths", ""+n.getDeaths()));
            if (Debug.debug) Debug.print(this, "adding statechange ("+a+",this,killdeaths,"+n.getKillDeaths()+")");
            vanilla.addStateChange(e, new PluginStateChange(a, this, "killdeaths", ""+n.getKillDeaths()));
         }
      }
      return null;
   }

   public Map formatDetails(AssassinStateViewer asv)
   {
      Map m = new HashMap();
      m.put("# of Kills", ""+asv.getIntProperty(this, "kills"));
      m.put("# of Deaths", ""+asv.getIntProperty(this, "deaths"));
      m.put("Kills - Deaths", ""+asv.getIntProperty(this, "killdeaths"));
      return m;
   }

}


