/**
 * Class representing an assassin's score
 */

package org.ucam.srcf.assassins.strawberry;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.StateChange;

class AssassinScore {
	private int baseScore;
	private int postGameMod;
	private boolean bDirty;
	private boolean pDirty;
	private boolean fDirty;
	private AssassinViewer ass;

	AssassinScore(AssassinViewer whose, String pname, VarTracker vt) {
		ass = whose;
		bDirty = pDirty = fDirty = false;
		baseScore = vt.getIntProperty(ass, "BaseScore");
		postGameMod = vt.getIntProperty(ass, "PostGameMod");
	}

	public AssassinViewer getAssassinViewer() {
		return ass;
	}

	public void setBaseScore(int to) {
		bDirty=true;
		fDirty=true;
		baseScore = to;
	}
	public void setPostGameMod(int to) {
		pDirty=true;
		fDirty=true;
		postGameMod = to;
	}

	public int addBase(int amount) {
		baseScore += amount;
		bDirty = fDirty = true;
		return baseScore;
	}
	public int addPost(int amount) {
		postGameMod += amount;
		pDirty = fDirty = true;
		return postGameMod;
	}

	public int getBaseScore() {
		return baseScore;
	}
	public int getPostGameMod() {
		return postGameMod;
	}
	public int getFinalScore() {
		return baseScore + postGameMod;
	}

	public boolean isBaseDirty() {
		return bDirty;
	}
	public boolean isPostDirty() {
		return pDirty;
	}
	public boolean isFinalDirty() {
		return fDirty;
	}

	public void baseClean() {
		bDirty=false;
	}
	public void postClean() {
		pDirty=false;
	}
	public void finalClean() {
		fDirty=false;
	}

	public StateChange getBaseSC(Plugin p) {
		baseClean();
		return new PluginStateChange(ass, p, "BaseScore", String.valueOf(baseScore));
	}
	public StateChange getPostSC(Plugin p) {
		postClean();
		return new PluginStateChange(ass, p, "PostGameMod", String.valueOf(postGameMod));
	}
	public StateChange getFinalSC(Plugin p) {
		finalClean();
		return new PluginStateChange(ass, p, "FinalScore", String.valueOf(baseScore + postGameMod));
	}
}
