/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */
package org.ucam.srcf.assassins.tuttifrutti;

import java.util.List;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * Interface for generic targetting algorithms.
 */
public interface TargettingAlgorithm {

	/**
	 * Retrieve the canonical name for the algorithm.
	 * @return The name of the algorithm.
	 */
	public String getName();
	
	/**
	 * Retrieve a list of all assassins controlled by this algorithm.
	 * @return The list of assassins involved in the algorithm.
	 */
	public List getAssassinList();

	/**
	 * Retrieve a list of an assassin's targets, as known to the algorithm.
	 * @param a The assassin to query
	 * @return The list of assassins targetted by this assassin
	 */
	public List getTargets(AssassinViewer a);

	/**
	 * Reallocate targets (normally after removal of assassins).
	 * @return List of PluginStateChange objects specifying entarget/detarget state changes resulting from this algorithm
	 */
	public List reallocateTargets() throws AssassinSQLException;
	
	/**
	 * Assign a target to an assassin.
	 * @param ass The assassin to give the target to
	 * @param targ The target to give to the assassin
	 */
	public void giveTarget(AssassinViewer ass, AssassinViewer targ);
	public void takeTarget(AssassinViewer ass, AssassinViewer targ);
	/**
	 * Remove an assassin from the algorithm (e.g. at death)
	 * @param a The assassin to remove
	 */
	public void removeAssassin(AssassinViewer a);
	
	/**
	 * Add an assassin to the algorithm, but do not recalculate targets.
	 * @param a The assassin to add
	 * @see TargettingAlgorithm.reallocateTargets
	 */
	public void addAssassin(AssassinViewer a);

}
