/*
 * CUAG AutoUmpire System
 *
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.tuttifrutti;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Debug;

public class TargettingStructure {

	private LinkedList assassinList = new LinkedList();
	private HashMap targetSets = new HashMap();
	private HashMap assassinSets = new HashMap();


	public TargettingStructure() {
	}

	public TargettingStructure(byte[][] tmatrix, List alist) {
		int n = alist.size();
		for (Iterator i = alist.iterator(); i.hasNext();) {
			addAssassin((AssassinViewer) i.next());
		}
		for (int i = 0; i < n; i++) {

			for (int j = 0; j < i; j++) {
				/*if (Debug.debug) Debug.print(TargettingStructure.class, "checking targetting for "+i+" "+j);*/
				if (tmatrix[i][j] == 1) {
					if (Debug.debug) Debug.print(TargettingStructure.class, "Targetting");
					giveTarget((AssassinViewer) assassinList.get(i), (AssassinViewer) assassinList.get(j));
				}
					
				if (tmatrix[i][j] == -1) {
					if (Debug.debug) Debug.print(TargettingStructure.class, "Assassinating");
					giveTarget((AssassinViewer) assassinList.get(j), (AssassinViewer) assassinList.get(i));
				}
			}
		}
	}


	public byte[][] getTargMatrix() {
			int m = 0;
			int n = 0;
			int playernum = assassinList.size();
			byte[][] tmatrix = new byte[playernum][playernum];
			for (Iterator i = assassinList.iterator(); i.hasNext();) {
				Object curi = i.next();
				n=0;
				for (Iterator j = assassinList.iterator(); j.hasNext();) {
					Object curj = j.next();
					/*if (Debug.debug) Debug.print(TargettingStructure.class, "Matrix position "+m+" "+n);*/
					if (((Set)targetSets.get(curi)).contains(curj)) {
						tmatrix[m][n] = 1;
						tmatrix[n][m] = -1;
					}
					n++;
				}
				m++;
			}
			return tmatrix;
	}

	public int[][] getFloydMatrix() {
		byte[][] adjacency = getTargMatrix();
		int playernum = assassinList.size();
		int[][] distance = new int[playernum][playernum];
		for (int i = 0; i < playernum; i++) {
			for (int j = 0; j < playernum; j++) {
				distance[i][j] = playernum + 1;
				if (adjacency[i][j] != 0) distance[i][j] = 1;
				if (i == j) distance[i][j] = 0;
			}
		}

		for (int k = 0; k < playernum; k++) {
			for (int i = 0; i < playernum; i++) {
				for (int j = 0; j <= i; j++) {
					int through_k = distance[i][k] + distance[k][j];
					if (through_k < distance[i][j]) {
						distance[i][j] = through_k;
						distance[j][i] = through_k;
					}
				}
			}
		}
		return distance;
	}

	public Object clone() {
		if (Debug.debug) Debug.print(TargettingStructure.class, "cloning structure");
		TargettingStructure newStruct = new TargettingStructure();
		for (Iterator i = assassinList.iterator(); i.hasNext();) {
			newStruct.addAssassin((AssassinViewer) i.next());
		}
		if (Debug.debug) Debug.print(TargettingStructure.class, "added assassins");
		for (Iterator i = assassinList.iterator(); i.hasNext();) {
			AssassinViewer cur = (AssassinViewer) i.next();
			if (Debug.debug) Debug.print(TargettingStructure.class, "getting targets for "+cur);
			if (null != targetSets.get(cur)) for (Iterator j = ((Set) targetSets.get(cur)).iterator(); j.hasNext();) {
				AssassinViewer a = (AssassinViewer) j.next();
				if (Debug.debug) Debug.print(TargettingStructure.class, "got "+a);
				if (newStruct.isInvolved(a)) newStruct.giveTarget(cur, a);
			}
		}
		return newStruct;

	}

	public List getAssassinList() {
		return assassinList;
	}

	public void addAssassin(AssassinViewer a) {
		assassinList.add(a);
		targetSets.put(a, new HashSet());
		assassinSets.put(a, new HashSet());
	}

	public void removeAssassin(AssassinViewer a) {
		if (Debug.debug) Debug.print(TargettingStructure.class, "in removeAssassin, removing "+a);
		if (assassinList.contains(a)) {
			for (Iterator i = assassinList.iterator(); i.hasNext();) {
				AssassinViewer p = (AssassinViewer) i.next();
				((Set) targetSets.get(p)).remove(a);
				((Set) assassinSets.get(p)).remove(a);
			}
			assassinList.remove(assassinList.indexOf(a));
		}
	}

	public List getTargets(AssassinViewer a) {
		List tlist = new LinkedList();
		for (Iterator i = ((Set) targetSets.get(a)).iterator(); i.hasNext();) {
			tlist.add(i.next());
		}
		return tlist;
	}

	public List getAssassins(AssassinViewer a) {
		List alist = new LinkedList();
		for (Iterator i = ((Set) assassinSets.get(a)).iterator(); i.hasNext();) {
			alist.add(i.next());
		}
		return alist;
	}

	public void giveTarget(AssassinViewer ass, AssassinViewer tar) {
		((Set) targetSets.get(ass)).add(tar);
		((Set) assassinSets.get(tar)).add(ass);
	}
	public void takeTarget(AssassinViewer ass, AssassinViewer tar) {
		((Set) targetSets.get(ass)).remove(tar);
		((Set) assassinSets.get(tar)).remove(ass);
	}
	public boolean isInvolved(AssassinViewer a) {
		return assassinList.contains(a);
	}

	public boolean hasTarget(AssassinViewer ass, AssassinViewer tar) {
		return ((Set) targetSets.get(ass)).contains(tar);
	}

	public boolean isEmpty() {
		for (Iterator i = assassinList.iterator(); i.hasNext();) {
			if (!(getTargets((AssassinViewer) i.next()).isEmpty())) return false;
		}
		return true;
	}

	public List triList() {
		List tlist = new LinkedList();
		for (Iterator i = assassinList.iterator(); i.hasNext();) {
			AssassinViewer a = (AssassinViewer) i.next();
			for (Iterator j = getTargets(a).iterator(); j.hasNext();) {
				AssassinViewer b = (AssassinViewer) j.next();
				for (Iterator k = getTargets(b).iterator(); k.hasNext();) {
					AssassinViewer c = (AssassinViewer) k.next();
					if (hasTarget(a,c) || hasTarget(c,a)) {
						tlist.add(a);
						tlist.add(b);
						tlist.add(c);
					}
				}
			}
		}
		return tlist;
	}

}
